<?php
namespace Fiscal\Logic\Parametros;

use Fiscal\Entities\Msgfiscncm;
use Fiscal\Entities\Msgfiscprod;
use Fiscal\Entities\Msgfiscregra;
use \Fiscal\Logic\Parametros\Exceptions\ExcecaoParametros;
use Geral\Entities\Imposto;
use Geral\Util\Ambiente;
use Indeterminado\Entities\Paramcst;
use \Indeterminado\Repositories\Regrafiscal;
use \Geral\Util\DateUtil;
use \Fiscal\Entities\Msgcfopadrao;
use \Fiscal\Entities\Msgcfopncm;
use \Fiscal\Entities\Msgcfopregfisc;
use Setforms2\Shared\Registry;

class ParametrizacaoCentralizada{

    const ATIVADO                  = 'S';
    const DESATIVADO               = 'N';

    const TABLE_CFOP_PADRAO        = 'CFOPADRAOREGFIS';
    const TABLE_CFOP_NCM           = 'CFOPNCMREGRAFIS';
    const TABLE_CFOP_PRODUTO       = 'CFOPREGRAFISCAL';

    const TABLE_IMPOSTO_PADRAO     = 'DADOSFISCREGRA';
    const TABLE_IMPOSTO_NCM        = 'DADOSFISCNCM';
    const TABLE_IMPOSTO_PRODUTO    = 'DADOSFISCPROD';

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    public static function ativacaoParametrizacao($cdregrafisc, $nrorg, $idativo)
    {
        /** @var $regrafiscal \Indeterminado\Entities\Regrafiscal */
        $regrafiscal = \Indeterminado\Entities\Regrafiscal::findOneByCdregrafiscAndNrorg($cdregrafisc, $nrorg);

        if ($idativo == self::DESATIVADO) {
            $regrafiscal->setIdativregrafisc(self::DESATIVADO);
        } else {
            $regrafiscal->setIdativregrafisc(self::ATIVADO);
        }
        $regrafiscal->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function updateDadosFiscaisCSTporImposto(Paramcst $dadosFiscais){
        $paramcst = Paramcst::findByCdimpostoAndCdsitutrib($dadosFiscais->getCdimposto(), $dadosFiscais->getCdsitutrib());

        /** @var $pcst Paramcst*/
        foreach($paramcst as $pcst){
            $pcst->setIduticdfiscal1($dadosFiscais->getIduticdfiscal1());
            $pcst->setIduticdfiscal2($dadosFiscais->getIduticdfiscal2());
            $pcst->setIduticdfiscal3($dadosFiscais->getIduticdfiscal3());
            $pcst->setIdutitipcredito($dadosFiscais->getIdutitipcredito());
            $pcst->setIdutialiqqtcst($dadosFiscais->getIdutialiqqtcst());

            $pcst->setCdoperultatu(Ambiente::getCdOperador());
            $pcst->setDtultatu(DateUtil::getDataAtual());
            $pcst->setNrorgultatu(Ambiente::getNrorg());

            $pcst->persist();
        }

        Registry::getEntityManager()->flush();
    }

    public static function insertMensagemCfop($table, $parametrizacao, $oper_interestadual, $ncm, $produto,
        $codigo, $descricao, $ativo, $organizacao, $org_inclusao, $org_atualizacao, $operador_inclusao,
        $operador_atualizacao, $data_inclusao, $data_atualizacao)
    {
        if ($table == self::TABLE_CFOP_PADRAO) {
            /** @var $mensagem Msgcfopadrao */
            $mensagem = new Msgcfopadrao();
            $mensagem->setCdregrafisc($parametrizacao);
            $mensagem->setIdcfopinterest($oper_interestadual);
            $mensagem->setNrseqmsg($codigo);
            $mensagem->setDsmensagem($descricao);
            $mensagem->setNrorg($organizacao);
            $mensagem->setIdativo($ativo);
            $mensagem->setDtinclusao(self::formataData($data_inclusao));
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
            $mensagem->setNrorginclusao($org_inclusao);
            $mensagem->setCdoperinclusao($operador_inclusao);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
        }
        elseif ($table == self::TABLE_CFOP_NCM) {
            /** @var $mensagem Msgcfopncm */
            $mensagem = new Msgcfopncm();
            $mensagem->setCdregrafisc($parametrizacao);
            $mensagem->setIdcfopinterest($oper_interestadual);
            $mensagem->setCdclasfisc($ncm);
            $mensagem->setNrseqmsg($codigo);
            $mensagem->setDsmensagem($descricao);
            $mensagem->setNrorg($organizacao);
            $mensagem->setIdativo($ativo);
            $mensagem->setDtinclusao(self::formataData($data_inclusao));
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
            $mensagem->setNrorginclusao($org_inclusao);
            $mensagem->setCdoperinclusao($operador_inclusao);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);

        }
        elseif ($table == self::TABLE_CFOP_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = new Msgcfopregfisc();
            $mensagem->setCdregrafisc($parametrizacao);
            $mensagem->setIdcfopinterest($oper_interestadual);
            $mensagem->setCdproduto($produto);
            $mensagem->setNrseqmsg($codigo);
            $mensagem->setDsmensagem($descricao);
            $mensagem->setNrorg($organizacao);
            $mensagem->setIdativo($ativo);
            $mensagem->setDtinclusao(self::formataData($data_inclusao));
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
            $mensagem->setNrorginclusao($org_inclusao);
            $mensagem->setCdoperinclusao($operador_inclusao);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
        }
        $mensagem->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();

    }

    public static function updateMensagemCfop($table, $parametrizacao, $oper_interestadual, $ncm, $produto,
        $codigo, $descricao, $ativo, $organizacao, $org_atualizacao, $operador_atualizacao, $data_atualizacao)
    {
        if ($table == self::TABLE_CFOP_PADRAO) {
            /** @var $mensagem Msgcfopadrao */
            $mensagem = Msgcfopadrao::findOneByCdregrafiscAndIdcfopinterestAndNrseqmsgAndNrorg(
                $parametrizacao, $oper_interestadual, $codigo, $organizacao
            );
            $mensagem->setDsmensagem($descricao);
            $mensagem->setIdativo($ativo);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
        }
        elseif ($table == self::TABLE_CFOP_NCM) {

            /** @var $mensagem Msgcfopncm */
            $mensagem = Msgcfopncm::findOneByCdregrafiscAndCdclasfiscAndIdcfopinterestAndNrseqmsgAndNrorg(
                                                    $parametrizacao, $ncm, $oper_interestadual, $codigo, $organizacao);

            $mensagem->setDsmensagem($descricao);
            $mensagem->setIdativo($ativo);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
            $mensagem->setDtultatu(self::formataData($data_atualizacao));

        }
        elseif ($table == self::TABLE_CFOP_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = Msgcfopregfisc::findOneByCdregrafiscAndCdprodutoAndIdcfopinterestAndNrseqmsgAndNrorg(
                $parametrizacao, $produto, $oper_interestadual, $codigo, $organizacao
            );
            $mensagem->setDsmensagem($descricao);
            $mensagem->setIdativo($ativo);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
        }
        $mensagem->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function deleteMensagemCfop($table, $parametrizacao, $oper_interestadual, $ncm, $produto,
        $codigo, $organizacao)
    {
        if ($table == self::TABLE_CFOP_PADRAO) {
             /** @var $mensagem Msgcfopadrao */
            $mensagem = Msgcfopadrao::findOneByCdregrafiscAndIdcfopinterestAndNrseqmsgAndNrorg(
                $parametrizacao, $oper_interestadual, $codigo, $organizacao
            );
        }
        elseif ($table == self::TABLE_CFOP_NCM) {
            /** @var $mensagem Msgcfopncm */
            $mensagem = Msgcfopncm::findOneByCdregrafiscAndCdclasfiscAndIdcfopinterestAndNrseqmsgAndNrorg(
                $parametrizacao, $ncm, $oper_interestadual, $codigo, $organizacao
            );
        }
        elseif ($table == self::TABLE_CFOP_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = Msgcfopregfisc::findOneByCdregrafiscAndCdprodutoAndIdcfopinterestAndNrseqmsgAndNrorg(
                $parametrizacao, $produto, $oper_interestadual, $codigo, $organizacao
            );
        }
        $mensagem->delete();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function deleteMensagensCfopCascade($table, $parametrizacao, $oper_interestadual, $ncm,
        $produto, $organizacao)
    {
        if ($table == self::TABLE_CFOP_PADRAO) {
            /** @var $men Msgcfopadrao */
            $mensagem = Msgcfopadrao::findByCdregrafiscAndIdcfopinterestAndNrorg(
                $parametrizacao, $oper_interestadual, $organizacao
            );
        }
        elseif ($table == self::TABLE_CFOP_NCM) {
            /** @var $mensagem Msgcfopncm */
            $mensagem = Msgcfopncm::findByCdregrafiscAndCdclasfiscAndIdcfopinterestAndNrorg(
                $parametrizacao, $ncm, $oper_interestadual, $organizacao
            );
        }
        elseif ($table == self::TABLE_CFOP_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = Msgcfopregfisc::findByCdregrafiscAndCdprodutoAndIdcfopinterestAndNrorg(
                $parametrizacao, $produto, $oper_interestadual, $organizacao
            );
        }
        foreach ($mensagem as $men) {
            $men->delete();
        }
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function insertMensagemImposto($table, $parametrizacao, $imposto, $codigo_dados_fisc, $ncm,
        $produto, $codigo, $descricao, $ativo, $organizacao, $org_inclusao, $org_atualizacao, $operador_inclusao,
        $operador_atualizacao, $data_inclusao, $data_atualizacao)
    {
        if ($table == self::TABLE_IMPOSTO_PADRAO) {
            /** @var $mensagem Msgfiscregra */
            $mensagem = new Msgfiscregra();
            $mensagem->setCdregrafisc($parametrizacao);
            $mensagem->setCdimposto($imposto);
            $mensagem->setNrseqdadosfisc($codigo_dados_fisc);
            $mensagem->setNrseqmsg($codigo);
            $mensagem->setDsmensagem($descricao);
            $mensagem->setNrorg($organizacao);
            $mensagem->setIdativo($ativo);
            $mensagem->setDtinclusao(self::formataData($data_inclusao));
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
            $mensagem->setNrorginclusao($org_inclusao);
            $mensagem->setCdoperinclusao($operador_inclusao);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
        }
        elseif ($table == self::TABLE_IMPOSTO_NCM) {
            /** @var $mensagem Msgfiscncm */
            $mensagem = new Msgfiscncm();
            $mensagem->setCdregrafisc($parametrizacao);
            $mensagem->setCdimposto($imposto);
            $mensagem->setNrseqdadosfisc($codigo_dados_fisc);
            $mensagem->setCdclasfisc($ncm);
            $mensagem->setNrseqmsg($codigo);
            $mensagem->setDsmensagem($descricao);
            $mensagem->setNrorg($organizacao);
            $mensagem->setIdativo($ativo);
            $mensagem->setDtinclusao(self::formataData($data_inclusao));
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
            $mensagem->setNrorginclusao($org_inclusao);
            $mensagem->setCdoperinclusao($operador_inclusao);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
        }
        elseif ($table == self::TABLE_IMPOSTO_PRODUTO) {
            /** @var $mensagem Msgfiscprod */
            $mensagem = new Msgfiscprod();
            $mensagem->setCdregrafisc($parametrizacao);
            $mensagem->setCdimposto($imposto);
            $mensagem->setNrseqdadosfisc($codigo_dados_fisc);
            $mensagem->setCdproduto($produto);
            $mensagem->setNrseqmsg($codigo);
            $mensagem->setDsmensagem($descricao);
            $mensagem->setNrorg($organizacao);
            $mensagem->setIdativo($ativo);
            $mensagem->setDtinclusao(self::formataData($data_inclusao));
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
            $mensagem->setNrorginclusao($org_inclusao);
            $mensagem->setCdoperinclusao($operador_inclusao);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
        }
        $mensagem->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function updateMensagemImposto($table, $parametrizacao, $imposto, $codigo_dados_fisc, $ncm,
        $produto, $codigo, $descricao, $ativo, $organizacao, $org_atualizacao, $operador_atualizacao, $data_atualizacao)
    {
        if ($table == self::TABLE_IMPOSTO_PADRAO) {
            /** @var $mensagem Msgcfopadrao */
            $mensagem = Msgfiscregra::findOneByCdregrafiscAndCdimpostoAndNrseqdadosfiscAndNrseqmsgAndNrorg(
                $parametrizacao,$imposto,$codigo_dados_fisc, $codigo, $organizacao
            );
            $mensagem->setDsmensagem($descricao);
            $mensagem->setIdativo($ativo);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
        }
        elseif ($table == self::TABLE_IMPOSTO_NCM) {
            /** @var $mensagem Msgcfopncm */
            $mensagem = Msgfiscncm::findOneByCdregrafiscAndCdclasfiscAndCdimpostoAndNrseqdadosfiscAndNrseqmsgAndNrorg(
                $parametrizacao, $ncm, $imposto, $codigo_dados_fisc, $codigo, $organizacao
            );
            $mensagem->setDsmensagem($descricao);
            $mensagem->setIdativo($ativo);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
        }
        elseif ($table == self::TABLE_IMPOSTO_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = Msgfiscprod::findOneByCdregrafiscAndCdprodutoAndCdimpostoAndNrseqdadosfiscAndNrseqmsgAndNrorg(
                $parametrizacao, $produto, $imposto, $codigo_dados_fisc, $codigo, $organizacao
            );
            $mensagem->setDsmensagem($descricao);
            $mensagem->setIdativo($ativo);
            $mensagem->setNrorgultatu($org_atualizacao);
            $mensagem->setCdoperultatu($operador_atualizacao);
            $mensagem->setDtultatu(self::formataData($data_atualizacao));
        }
        $mensagem->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function deleteMensagemImposto($table, $parametrizacao, $imposto, $codigo_dados_fisc, $ncm, $produto,
        $codigo, $organizacao)
    {
        if ($table == self::TABLE_IMPOSTO_PADRAO) {
            /** @var $mensagem Msgcfopadrao */
            $mensagem = Msgfiscregra::findOneByCdregrafiscAndCdimpostoAndNrseqdadosfiscAndNrseqmsgAndNrorg(
                $parametrizacao,$imposto,$codigo_dados_fisc, $codigo, $organizacao
            );
        }
        elseif ($table == self::TABLE_IMPOSTO_NCM) {
            /** @var $mensagem Msgcfopncm */
            $mensagem = Msgfiscncm::findOneByCdregrafiscAndCdclasfiscAndCdimpostoAndNrseqdadosfiscAndNrseqmsgAndNrorg(
                $parametrizacao, $ncm, $imposto, $codigo_dados_fisc, $codigo, $organizacao
            );
        }
        elseif ($table == self::TABLE_IMPOSTO_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = Msgfiscprod::findOneByCdregrafiscAndCdprodutoAndCdimpostoAndNrseqdadosfiscAndNrseqmsgAndNrorg(
                $parametrizacao, $produto, $imposto, $codigo_dados_fisc, $codigo, $organizacao
            );
        }
        $mensagem->delete();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    public static function deleteMensagensImpostoCascade($table, $parametrizacao, $imposto, $codigo_dados_fisc, $ncm,
        $produto, $organizacao)
    {
        if ($table == self::TABLE_IMPOSTO_PADRAO) {
            /** @var $mensagem Msgcfopadrao */
            $mensagem = Msgfiscregra::findByCdregrafiscAndCdimpostoAndNrseqdadosfiscAndNrorg(
                $parametrizacao,$imposto,$codigo_dados_fisc, $organizacao
            );
        } elseif ($table == self::TABLE_IMPOSTO_NCM) {
            /** @var $mensagem Msgcfopncm */
            $mensagem = Msgfiscncm::findByCdregrafiscAndCdclasfiscAndCdimpostoAndNrseqdadosfiscAndNrorg(
                $parametrizacao, $ncm, $imposto, $codigo_dados_fisc, $organizacao
            );
        } elseif ($table == self::TABLE_IMPOSTO_PRODUTO) {
            /** @var $mensagem Msgcfopregfisc */
            $mensagem = Msgfiscprod::findByCdregrafiscAndCdprodutoAndCdimpostoAndNrseqdadosfiscAndNrorg(
                $parametrizacao, $produto, $imposto, $codigo_dados_fisc, $organizacao
            );
        }
        foreach ($mensagem as $men) {
            $men->delete();
        }
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }
}