import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Button,
} from '@zeedhi/common';

export class Pme03000EmissaoCardPeriodoController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  private relatoriosEmissaoCardapio = Loader.getInstance('RelatoriosEmissaoCardController');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');

    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/emissaoCardPeriodo', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setParams(row: IDictionary) {
    const paramsCard = this.relatoriosEmissaoCardapio.getParamsCard();
    
    row.CDTIPOCARD = row.TIPOCARD.CDTIPOCARD;
    row.NMTIPOCARD = row.TIPOCARD.NMTIPOCARD;
    row.NRCARDPME = paramsCard?.NRCARDPME ?? this.elaboracaoCardapio.getFilterData.NRCARDPME;
    row.NMFILIAGRE = row.UNIDADE ? row.UNIDADE.NMFILIAL : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE.CDFILIAL : null;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.LISTASERV = '';
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const periodoDe = row.PERIODODE;
    const periodoAte = row.PERIODOATE;
    form.validate();
    if (periodoDe === '' || periodoAte === '') this.utilities.showAlert('alertError', 'pme::periodoObrig', 'error');
    else if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(periodoDe, periodoAte) && this.utilities.dateIsValid(dtDe, dtAte)) {
        if (this.utilities.dateIsValid(periodoDe, dtDe) && this.utilities.dateIsValid(dtAte, periodoAte)) valid = true;
        else this.utilities.showAlert('alertError', 'pme::dtCompreendidaPerCard', 'error');
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }
}
